from __future__ import absolute_import, division, print_function

import stripe


class TestNestedResourceClassMethods(object):
    @stripe.api_resources.abstract.nested_resource_class_methods(
        'nested',
        operations=['create', 'retrieve', 'update', 'delete', 'list']
    )
    class MainResource(stripe.api_resources.abstract.APIResource):
        OBJECT_NAME = 'mainresource'

    def test_create_nested(self, request_mock):
        request_mock.stub_request(
            'post',
            '/v1/mainresources/id/nesteds',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'bar',
            }
        )
        nested_resource = self.MainResource.create_nested('id', foo='bar')
        request_mock.assert_requested(
            'post',
            '/v1/mainresources/id/nesteds',
            {
                'foo': 'bar'
            },
            None
        )
        assert nested_resource.foo == 'bar'

    def test_retrieve_nested(self, request_mock):
        request_mock.stub_request(
            'get',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'bar',
            }
        )
        nested_resource = self.MainResource.retrieve_nested('id', 'nested_id')
        request_mock.assert_requested(
            'get',
            '/v1/mainresources/id/nesteds/nested_id',
            {},
            None
        )
        assert nested_resource.foo == 'bar'

    def test_modify_nested(self, request_mock):
        request_mock.stub_request(
            'post',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'baz',
            }
        )
        nested_resource = self.MainResource.modify_nested('id', 'nested_id',
                                                          foo='baz')
        request_mock.assert_requested(
            'post',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'foo': 'baz'
            },
            None
        )
        assert nested_resource.foo == 'baz'

    def test_delete_nested(self, request_mock):
        request_mock.stub_request(
            'delete',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'deleted': True,
            }
        )
        nested_resource = self.MainResource.delete_nested('id', 'nested_id')
        request_mock.assert_requested(
            'delete',
            '/v1/mainresources/id/nesteds/nested_id',
            {},
            None
        )
        assert nested_resource.deleted is True

    def test_list_nesteds(self, request_mock):
        request_mock.stub_request(
            'get',
            '/v1/mainresources/id/nesteds',
            {
                'object': 'list',
                'data': [],
            }
        )
        nested_resource = self.MainResource.list_nesteds('id')
        request_mock.assert_requested(
            'get',
            '/v1/mainresources/id/nesteds',
            {},
            None
        )
        assert isinstance(nested_resource.data, list)



from __future__ import absolute_import, division, print_function

import stripe


class TestDeletableAPIResource(object):
    class MyDeletable(stripe.api_resources.abstract.DeletableAPIResource):
        OBJECT_NAME = 'mydeletable'

    def test_delete(self, request_mock):
        request_mock.stub_request(
            'delete',
            '/v1/mydeletables/mid',
            {
                'id': 'mid',
                'deleted': True,
            },
            rheaders={'request-id': 'req_id'}
        )

        obj = self.MyDeletable.construct_from({
            'id': 'mid'
        }, 'mykey')

        assert obj is obj.delete()
        request_mock.assert_requested(
            'delete',
            '/v1/mydeletables/mid',
            {}
        )
        assert obj.deleted is True
        assert obj.id == 'mid'

        assert obj.last_response is not None
        assert obj.last_response.request_id == 'req_id'



from __future__ import absolute_import, division, print_function

import datetime
import json
import pickle
from copy import copy, deepcopy

import pytest

import stripe
from stripe import six


SAMPLE_INVOICE = json.loads("""
{
  "amount_due": 1305,
  "attempt_count": 0,
  "attempted": true,
  "charge": "ch_wajkQ5aDTzFs5v",
  "closed": true,
  "customer": "cus_osllUe2f1BzrRT",
  "date": 1338238728,
  "discount": null,
  "ending_balance": 0,
  "id": "in_t9mHb2hpK7mml1",
  "livemode": false,
  "next_payment_attempt": null,
  "object": "invoice",
  "paid": true,
  "period_end": 1338238728,
  "period_start": 1338238716,
  "starting_balance": -8695,
  "subtotal": 10000,
  "total": 10000,
  "lines": {
    "invoiceitems": [],
    "prorations": [],
    "subscriptions": [
      {
        "plan": {
          "interval": "month",
          "object": "plan",
          "identifier": "expensive",
          "currency": "usd",
          "livemode": false,
          "amount": 10000,
          "name": "Expensive Plan",
          "trial_period_days": null,
          "id": "expensive"
        },
        "period": {
          "end": 1340917128,
          "start": 1338238728
        },
        "amount": 10000
      }
    ]
  }
}
""")


class TestStripeObject(object):
    def test_initializes_with_parameters(self):
        obj = stripe.stripe_object.StripeObject(
            'foo', 'bar', myparam=5, yourparam='boo')

        assert obj.id == 'foo'
        assert obj.api_key == 'bar'

    def test_access(self):
        obj = stripe.stripe_object.StripeObject('myid', 'mykey', myparam=5)

        # Empty
        with pytest.raises(AttributeError):
            obj.myattr
        with pytest.raises(KeyError):
            obj['myattr']
        assert obj.get('myattr', 'def') == 'def'
        assert obj.get('myattr') is None

        # Setters
        obj.myattr = 'myval'
        obj['myitem'] = 'itval'
        assert obj.setdefault('mydef', 'sdef') == 'sdef'

        # Getters
        assert obj.setdefault('myattr', 'sdef') == 'myval'
        assert obj.myattr == 'myval'
        assert obj['myattr'] == 'myval'
        assert obj.get('myattr') == 'myval'

        assert sorted(obj.keys()) == ['id', 'myattr', 'mydef', 'myitem']

        assert sorted(obj.values()) == ['itval', 'myid', 'myval', 'sdef']

        # Illegal operations
        with pytest.raises(ValueError):
            obj.foo = ''

    def test_refresh_from(self, mocker):
        obj = stripe.stripe_object.StripeObject.construct_from({
            'foo': 'bar',
            'trans': 'me',
        }, 'mykey')

        assert obj.api_key == 'mykey'
        assert obj.foo == 'bar'
        assert obj['trans'] == 'me'
        assert obj.stripe_version is None
        assert obj.stripe_account is None
        assert obj.last_response is None

        last_response = mocker.Mock()
        obj.refresh_from(
            {
                'foo': 'baz',
                'johnny': 5,
            }, 'key2',
            stripe_version='2017-08-15',
            stripe_account='acct_foo',
            last_response=last_response
        )

        assert obj.johnny == 5
        assert obj.foo == 'baz'
        with pytest.raises(AttributeError):
            obj.trans
        assert obj.api_key == 'key2'
        assert obj.stripe_version == '2017-08-15'
        assert obj.stripe_account == 'acct_foo'
        assert obj.last_response == last_response

        obj.refresh_from({
            'trans': 4,
            'metadata': {'amount': 42}
        }, 'key2', True)

        assert obj.foo == 'baz'
        assert obj.trans == 4

    def test_passing_nested_refresh(self):
        obj = stripe.stripe_object.StripeObject.construct_from({
            'foos': {
                'type': 'list',
                'data': [
                    {'id': 'nested'}
                ],
            }
        }, 'key', stripe_account='acct_foo')

        nested = obj.foos.data[0]

        assert obj.api_key == 'key'
        assert nested.id == 'nested'
        assert nested.api_key == 'key'
        assert nested.stripe_account == 'acct_foo'

    def test_refresh_from_nested_object(self):
        obj = stripe.stripe_object.StripeObject.construct_from(
            SAMPLE_INVOICE, 'key')

        assert len(obj.lines.subscriptions) == 1
        assert isinstance(obj.lines.subscriptions[0],
                          stripe.stripe_object.StripeObject)
        assert obj.lines.subscriptions[0].plan.interval == 'month'

    def test_to_json(self):
        obj = stripe.stripe_object.StripeObject.construct_from(
            SAMPLE_INVOICE, 'key')

        self.check_invoice_data(json.loads(str(obj)))

    def check_invoice_data(self, data):
        # Check rough structure
        assert len(list(data.keys())) == 20
        assert len(list(data['lines'].keys())) == 3
        assert len(data['lines']['invoiceitems']) == 0
        assert len(data['lines']['subscriptions']) == 1

        # Check various data types
        assert data['date'] == 1338238728
        assert data['next_payment_attempt'] is None
        assert data['livemode'] is False
        assert data['lines']['subscriptions'][0]['plan']['interval'] == 'month'

    def test_repr(self):
        obj = stripe.stripe_object.StripeObject(
            'foo', 'bar', myparam=5)

        obj['object'] = u'\u4e00boo\u1f00'
        obj.date = datetime.datetime.fromtimestamp(1511136000)

        res = repr(obj)

        if six.PY2:
            res = six.text_type(repr(obj), 'utf-8')

        assert u'<StripeObject \u4e00boo\u1f00' in res
        assert u'id=foo' in res
        assert u'"date": 1511136000' in res

    def test_pickling(self):
        obj = stripe.stripe_object.StripeObject(
            'foo', 'bar', myparam=5)

        obj['object'] = 'boo'
        obj.refresh_from(
            {
                'fala': 'lalala',
                # ensures that empty strings are correctly unpickled in Py3
                'emptystring': '',
            },
            api_key='bar', partial=True
        )

        assert obj.fala == 'lalala'

        pickled = pickle.dumps(obj)
        newobj = pickle.loads(pickled)

        assert newobj.id == 'foo'
        assert newobj.api_key == 'bar'
        assert newobj['object'] == 'boo'
        assert newobj.fala == 'lalala'
        assert newobj.emptystring == ''

    def test_deletion(self):
        obj = stripe.stripe_object.StripeObject('id', 'key')

        obj.coupon = "foo"
        assert obj.coupon == 'foo'

        del obj.coupon
        with pytest.raises(AttributeError):
            obj.coupon

        obj.refresh_from({'coupon': 'foo'}, api_key='bar', partial=True)
        assert obj.coupon == 'foo'

    def test_copy(self):
        nested = stripe.stripe_object.StripeObject.construct_from({
            'value': 'bar',
        }, 'mykey')
        obj = stripe.stripe_object.StripeObject.construct_from({
            'empty': '',
            'value': 'foo',
            'nested': nested,
        }, 'mykey', stripe_account='myaccount')

        copied = copy(obj)

        assert copied.empty == ''
        assert copied.value == 'foo'
        assert copied.nested.value == 'bar'

        assert copied.api_key == 'mykey'
        assert copied.stripe_account == 'myaccount'

        # Verify that we're not deep copying nested values.
        assert id(nested) == id(copied.nested)

    def test_deepcopy(self):
        nested = stripe.stripe_object.StripeObject.construct_from({
            'value': 'bar',
        }, 'mykey')
        obj = stripe.stripe_object.StripeObject.construct_from({
            'empty': '',
            'value': 'foo',
            'nested': nested,
        }, 'mykey', stripe_account='myaccount')

        copied = deepcopy(obj)

        assert copied.empty == ''
        assert copied.value == 'foo'
        assert copied.nested.value == 'bar'

        assert copied.api_key == 'mykey'
        assert copied.stripe_account == 'myaccount'

        # Verify that we're actually deep copying nested values.
        assert id(nested) != id(copied.nested)

    def test_serialize_empty_string_unsets(self):
        class SerializeToEmptyString(stripe.stripe_object.StripeObject):
            def serialize(self, previous):
                return ''

        nested = SerializeToEmptyString.construct_from({
            'value': 'bar',
        }, 'mykey')
        obj = stripe.stripe_object.StripeObject.construct_from({
            'nested': nested,
        }, 'mykey')

        assert obj.serialize(None) == {'nested': ''}



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'we_123'


class TestWebhookEndpoint(object):
    def test_is_listable(self, request_mock):
        resources = stripe.WebhookEndpoint.list()
        request_mock.assert_requested(
            'get',
            '/v1/webhook_endpoints'
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.WebhookEndpoint)

    def test_is_retrievable(self, request_mock):
        resource = stripe.WebhookEndpoint.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/webhook_endpoints/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.WebhookEndpoint)

    def test_is_creatable(self, request_mock):
        resource = stripe.WebhookEndpoint.create(
            enabled_events=['charge.succeeded'],
            url='https://stripe.com'
        )
        request_mock.assert_requested(
            'post',
            '/v1/webhook_endpoints'
        )
        assert isinstance(resource, stripe.WebhookEndpoint)

    def test_is_saveable(self, request_mock):
        resource = stripe.WebhookEndpoint.retrieve(TEST_RESOURCE_ID)
        resource.enabled_events = ['charge.succeeded']
        resource.save()
        request_mock.assert_requested(
            'post',
            '/v1/webhook_endpoints/%s' % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = stripe.WebhookEndpoint.modify(
            TEST_RESOURCE_ID,
            enabled_events=['charge.succeeded']
        )
        request_mock.assert_requested(
            'post',
            '/v1/webhook_endpoints/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.WebhookEndpoint)

    def test_is_deletable(self, request_mock):
        resource = stripe.WebhookEndpoint.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            'delete',
            '/v1/webhook_endpoints/%s' % TEST_RESOURCE_ID
        )
        assert resource.deleted is True



from __future__ import absolute_import, division, print_function

import stripe


class TestCheckoutSession(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.CheckoutSession.create(
            allowed_source_types=['card'],
            cancel_url='https://stripe.com/cancel',
            client_reference_id='1234',
            line_items=[
                {
                    'amount': 123,
                    'currency': 'usd',
                    'description': 'item 1',
                    'images': [
                        'https://stripe.com/img1',
                    ],
                    'name': 'name',
                    'quantity': 2,
                },
            ],
            payment_intent_data={
                'receipt_email': 'test@stripe.com',
            },
            success_url='https://stripe.com/success'
        )
        request_mock.assert_requested(
            'post',
            '/v1/checkout_sessions'
        )
        assert isinstance(resource, stripe.CheckoutSession)



from __future__ import absolute_import, division, print_function

import stripe


class TestAccountLink(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.AccountLink.create(
            account='acct_123',
            failure_url='https://stripe.com/failure',
            success_url='https://stripe.com/success',
            type='custom_account_verification'
        )
        request_mock.assert_requested(
            'post',
            '/v1/account_links'
        )
        assert isinstance(resource, stripe.AccountLink)



from __future__ import absolute_import, division, print_function

import pytest

import stripe


class TestAPIResource(object):
    class MyResource(stripe.api_resources.abstract.APIResource):
        OBJECT_NAME = 'myresource'

    def test_retrieve_and_refresh(self, request_mock):
        url = '/v1/myresources/foo%2A'
        request_mock.stub_request(
            'get',
            url,
            {
                'id': 'foo2',
                'bobble': 'scrobble',
            },
            rheaders={'request-id': 'req_id'}
        )

        res = self.MyResource.retrieve('foo*', myparam=5)

        request_mock.assert_requested(
            'get',
            url,
            {
                'myparam': 5,
            },
            None
        )
        assert res.bobble == 'scrobble'
        assert res.id == 'foo2'
        assert res.api_key == 'sk_test_123'

        assert res.last_response is not None
        assert res.last_response.request_id == 'req_id'

        url = '/v1/myresources/foo2'
        request_mock.stub_request(
            'get',
            url,
            {
                'frobble': 5,
            }
        )

        res = res.refresh()

        request_mock.assert_requested(
            'get',
            url,
            {
                'myparam': 5,
            },
            None
        )
        assert res.frobble == 5
        with pytest.raises(KeyError):
            res['bobble']

    def test_convert_to_stripe_object(self):
        sample = {
            'foo': 'bar',
            'adict': {
                'object': 'charge',
                'id': 42,
                'amount': 7,
            },
            'alist': [
                {
                    'object': 'customer',
                    'name': 'chilango'
                }
            ]
        }

        converted = stripe.util.convert_to_stripe_object(
            sample, 'akey', None, None)

        # Types
        assert isinstance(converted, stripe.stripe_object.StripeObject)
        assert isinstance(converted.adict, stripe.Charge)
        assert len(converted.alist) == 1
        assert isinstance(converted.alist[0], stripe.Customer)

        # Values
        assert converted.foo == 'bar'
        assert converted.adict.id == 42
        assert converted.alist[0].name == 'chilango'

        # Stripping
        # TODO: We should probably be stripping out this property
        # self.assertRaises(AttributeError, getattr, converted.adict, 'object')

    def test_raise_on_incorrect_id_type(self):
        for obj in [None, 1, 3.14, dict(), list(), set(), tuple(), object()]:
            with pytest.raises(stripe.error.InvalidRequestError):
                self.MyResource.retrieve(obj)



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'btcrcv_123'


class TestBitcoinReceiver(object):
    def construct_resource(self, **params):
        res_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'bitcoin_receiver',
            'metadata': {},
        }
        res_dict.update(params)
        return stripe.BitcoinReceiver.construct_from(res_dict, stripe.api_key)

    def test_has_customer_instance_url(self):
        resource = self.construct_resource(customer='cus_123')
        assert resource.instance_url() == \
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID

    def test_has_receiver_instance_url(self):
        resource = self.construct_resource()
        assert resource.instance_url() == \
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID

    def test_is_listable(self, request_mock):
        resources = stripe.BitcoinReceiver.list()
        request_mock.assert_requested(
            'get',
            '/v1/bitcoin/receivers'
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.BitcoinReceiver)

    def test_is_retrievable(self, request_mock):
        resource = stripe.BitcoinReceiver.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.BitcoinReceiver)



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'prv_123'


class TestReview(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Review.list()
        request_mock.assert_requested(
            'get',
            '/v1/reviews'
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Review)

    def test_is_retrievable(self, request_mock):
        resource = stripe.Review.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/reviews/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Review)

    def test_is_approveable(self, request_mock):
        resource = stripe.Review.retrieve(TEST_RESOURCE_ID)
        resource.approve()
        request_mock.assert_requested(
            'post',
            '/v1/reviews/%s/approve' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Review)



import os
import sys
from codecs import open
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass into pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = '-n auto'

    def run_tests(self):
        import shlex
        import pytest
        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


here = os.path.abspath(os.path.dirname(__file__))

os.chdir(here)

with open(os.path.join(here, 'LONG_DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

version_contents = {}
with open(os.path.join(here, 'stripe', 'version.py'), encoding='utf-8') as f:
    exec(f.read(), version_contents)

setup(
    name='stripe',
    version=version_contents['VERSION'],
    description='Python bindings for the Stripe API',
    long_description=long_description,
    author='Stripe',
    author_email='support@stripe.com',
    url='https://github.com/stripe/stripe-python',
    license='MIT',
    keywords='stripe api payments',
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={'stripe': ['data/ca-certificates.crt']},
    zip_safe=False,
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"',
        'requests[security] >= 2.20; python_version < "3.0"',
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    tests_require=[
        'pytest >= 3.4',
        'pytest-mock >= 1.7',
        'pytest-xdist >= 1.22',
        'pytest-cov >= 2.5',
    ],
    cmdclass={'test': PyTest},
    project_urls={
        'Bug Tracker': 'https://github.com/stripe/stripe-python/issues',
        'Documentation': 'https://stripe.com/docs/api/python',
        'Source Code': 'https://github.com/stripe/stripe-python',
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'rdr_123'


class TestConnectionToken(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.terminal.ConnectionToken.create()
        request_mock.assert_requested(
            'post',
            '/v1/terminal/connection_tokens'
        )
        assert isinstance(resource, stripe.terminal.ConnectionToken)
